import Logger from "@home-gallery/logger";
import { ExtensionRegistry } from "./extensionRegistry.js";
import { DatabaseSchema, ExtractorSchema, QuerySchema } from "./pluginSchemas.js";
import { PluginLoader } from "./load/loader.js";
import { proxyRegisterForPlugin } from "./pluginRegistryProxy.js";
class PluginManager {
  log = Logger("pluginManager");
  config;
  context;
  loader;
  registry;
  constructor(config = {}, context = {}) {
    this.config = config;
    this.context = context;
    this.loader = new PluginLoader(config);
    this.registry = new ExtensionRegistry({
      "extractor": ExtractorSchema,
      "database": DatabaseSchema,
      "query": QuerySchema
    }, config.pluginManager?.disabled || []);
  }
  getApiVersion() {
    return "0.7";
  }
  getConfig() {
    return this.config;
  }
  createLogger(module) {
    return Logger(module);
  }
  getContext() {
    return this.context;
  }
  addPlugin(plugin) {
    return this.loader.addPlugin("direct.source", plugin);
  }
  async #initializePlugins() {
    const orderedPlugins = this.loader.getResolvedPlugins();
    for (let ctx of orderedPlugins) {
      const managerProxy = proxyRegisterForPlugin(this, this.registry, ctx.plugin);
      await ctx.plugin.initialize(managerProxy).then((factory) => {
        if (typeof factory == "object") {
          return this.registry.registerLegacyFactory(ctx.plugin, factory);
        }
      }).then(() => {
        ctx.initialized = true;
      }).catch((err) => {
        this.log.error(err, `Failed to initialize plugin ${ctx.plugin.name} (from ${ctx.file}): ${err}`);
      });
    }
  }
  async loadPlugins() {
    const files = this.config.pluginManager?.plugins || [];
    for (let file of files) {
      await this.loader.loadPlugin(file).catch((err) => {
        this.log.warn(err, `Failed to load plugin from file ${file}: ${err}`);
      });
    }
    const dirs = this.config.pluginManager?.dirs || [];
    for (let dir of dirs) {
      await this.loader.loadPluginDir(dir).catch((err) => {
        this.log.warn(err, `Failed to load plugins from dir ${dir}: ${err}`);
      });
    }
    await this.#initializePlugins();
  }
  async register(type, extension) {
    throw new Error("Illigal method call");
  }
  getExtensions() {
    return this.registry.getExtensions();
  }
  getPlugins() {
    const result = [];
    for (let extension of this.getExtensions()) {
      if (!result.includes(extension.plugin)) {
        result.push(extension.plugin);
      }
    }
    return result;
  }
}
export {
  PluginManager
};
//# sourceMappingURL=manager.js.map
